/* 
   - The Original Code is this file as it was released on September 22, 2003.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2003-2004
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
*/

const nsIPermissionManager = Components.interfaces.nsIPermissionManager;
const nsICookie = Components.interfaces.nsICookie;

const TOGGLE_BUTTON = 2;
const SESSION_BUTTON = 3;
const REDIRECT_BUTTON = 4;
const ASKME_BUTTON = 5;
const PREF_BUTTON = 7;
const ADD_BUTTON = 8;
const ALLOW_SESSION_ONLY = 8; // XXX see bug 225857 comment 7

var observerService;
var cookiemanager = null;
var permissionmanager = null;
var popupmanager = null;
var permissionsTree;
var permissions = [];
var deletedPermissions = [];
var lastPermissionSortColumn = "rawHost";
var lastPermissionSortAscending = true;
var entryBox = null;
var newHostField = null;
var permissionTypes = [ "cookie", "image", "popup", "object", "script", "frame", "stylesheet", "meta" ];
var managerTypes = [ "cookieManager", "imageManager", "popupManager", "pluginManager", "scriptManager", "frameManager", "stylesheetManager", "metaManager"];
var toolbarButtons = [ "blockButton", "allowButton", "toggleButton", "sessionButton", "redirectButton", "askmeButton", "deleteButton", "prefsButton", "addButton" ];
/***
  * Toolbar button arrays, this should ease the pain of enabling/disabling the buttons
  */
var defaultEnabledButtons    = [ false, false, true,  false, false, false,  true,  true, false ];
var popupEnabledButtons      = [ true,  true,  false, false, true,  true,   true,  true, false ];
var selectEnabledButtons     = [ false, false, false, false, false, false,  true, false, false ];
var inputEnabledButtons      = [ false, false, false, false, false, false, false, false, true  ];
var cookieEnabledButtons     = [ true,  true,  false,  true, false, false,  true,  true, false ];
var cookieDataEnabledButtons = [ false, false, false, false, false, false,  true,  true, false ];

var prefPanelNames;
var dialogType = "";
var typeID = Number(0);
var permissionsBundle, cookieBundle;
var gLastHost;
var gUpdatingBatch = "";
var gDateService = null;
var gDisplayCookieData = false;
var gPreventNewCookies = false;
var gCookieDataFromPrefPanel = false;
var gLastCookie = null;
var gUseStackedWindows = false;
var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
var pref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);

var gFileName = null;
var cbDatasource = null;
var prefPatternLeader = "multizilla.content-blocker.pattern.for.";
var patternTypeNumbers = [ 1, 3, 4, 5, 6, 7 ]; // skip cookies and popups
var gWindowScreenY, gWindowScreenX, gWindowHeight, gWindowWidth;


function Startup()
{
  observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  setupIDs(window.arguments[0], true);
  entryBox = document.getElementById("EntryBox");
  newHostField = entryBox.firstChild;
  permissionsTree = document.getElementById("permissionsTree");

  permissionmanager = Components.classes["@mozilla.org/permissionmanager;1"]
                                .getService(Components.interfaces.nsIPermissionManager);
  cookiemanager = Components.classes["@mozilla.org/cookiemanager;1"]
                            .getService(Components.interfaces.nsICookieManager);

  permissionsBundle = mzGetStringBundle("chrome://multiviews/locale/permissionManager.properties");
  cookieBundle = mzGetStringBundle("chrome://communicator/locale/wallet/CookieViewer.properties");

  if (!gDateService) {
    gDateService = Components.classes["@mozilla.org/intl/scriptabledateformat;1"]
                             .getService(Components.interfaces.nsIScriptableDateFormat);
  }
  prefPanelNames = permissionsBundle.GetStringFromName('prefPanelNames').split(',');
  document.getElementById('mac_button_spring').hidden = !(/Mac/.test(navigator.platform));
  // Initialize button array
  for (var i = 0; i < toolbarButtons.length; i++) {
    toolbarButtons[i] = document.getElementById(toolbarButtons[i]);
  }
  switchTree(dialogType);
  observerService.addObserver(permissionsReloadDisplay, "perm-changed", false);

  if (window.arguments.length > 1) {
    var prefill = window.arguments[1].replace(/www./i,'');
    var hostFound = isDuplicate(prefill);

    if (hostFound >= 0) {
      permissionsTree.currentIndex = hostFound;
      permissionsTree.view.selection.toggleSelect(permissionsTree.currentIndex);
    }
    else {
      newHostField.value = prefill;
      gLastHost = prefill;
    }
  }
  permissionsTree.focus();
  updateToolbarButtons();
  updateLastHost();
  sizeToContent();
  initWindowOpenType();
}

function setupIDs(arg, aFlag)
{
  dialogType = null; // Reset selected manager

  if (arg == "cookieManagerFromIcon") { // Called from cookie icon on status bar
    observerService.notifyObservers(null, "cookieIcon", "off");
    gDisplayCookieData = true;
    arg = "cookieManager";
  }
  if (aFlag) {
    for (typeID in managerTypes) {
      if (managerTypes[typeID] == arg)
        break;
    }
  }
  else {
    for (typeID in permissionTypes) {
      if (permissionTypes[typeID] == arg)
        break;
    }
  }
  if (arg == "mozilla-image-prefs") {
    typeID = 1; // Keep 'Images' active!
  }
  else if (arg == "mozilla-popup-prefs") {
    typeID = 2; // Keep 'Popups' active!
  }
  typeID = Number(typeID);
  dialogType = permissionTypes[typeID];
  // Select Category Button
  var button = document.getElementById(dialogType);
  button.checked = true;
  button.focus();

  if (!dialogType) {
    alert(permissionsBundle.GetStringFromName('unsupportedManagerSelected'));
    window.close();
  }
}

function Shutdown()
{
  observerService.removeObserver(permissionsReloadDisplay, "perm-changed");
  // Save last active pref panel
  var str = Components.classes["@mozilla.org/supports-string;1"]
                      .createInstance(Components.interfaces.nsISupportsString);
  str.data = managerTypes[typeID];
  pref.setComplexValue("multizilla.perm-manager.default-manager", Components.interfaces.nsISupportsString, str);
}

var permissionsReloadDisplay = {
  observe: function(subject, topic, state) {

    if (topic == gUpdatingBatch)
      return;
    if (topic == "perm-changed") {
      permissions.length = 0;
      if (lastPermissionSortColumn == "rawHost") {
        lastPermissionSortAscending = !lastPermissionSortAscending;
      }
      loadPermissions();
    }
  }
}

var permissionsTreeView = {
  rowCount : 0,
  setTree : function(tree){},
  getImageSrc : function(row,column) {},
  getProgressMode : function(row,column) {},
  getCellValue : function(row,column) {},
  getCellText : function(row,column) {
    var rv = "";

    if (column.id == "siteCol" || column == "siteCol") {
      rv = permissions[row].rawHost;
    } else if (gDisplayCookieData && (column.id == "nameCol" || column == "nameCol")) {
      rv = permissions[row].name;
    } else if (column.id == "statusCol" || column == "statusCol") {
      rv = gDisplayCookieData ? getStatusString(permissions[row].capability) : permissions[row].capability;
    }
    return rv;
  },
  getParentIndex : function(row) {return -1;},
  isSeparator : function(index) {return false;},
  isSorted: function() { return false;},
  isContainer : function(index) {return false;},
  selectionChanged: function() {
    updateToolbarButtons();
    updateLastHost();

    if (gDisplayCookieData)
      cookieSelected();
  },
  cycleHeader : function(aColId) {},
  getRowProperties : function(row, prop) {},
  getColumnProperties : function(column, prop) {},
  getCellProperties : function(row, column, prop) {},
  isEditable: function(row, column) { return true; }
 };

function Permission(number, host, rawHost, type, capability, name, value, isDomain, path, isSecure, expires, policy)
{
  this.number = number;
  this.host = host;
  this.rawHost = rawHost;
  this.type = type;
  this.capability = capability;
  // Cookie only data
  if (gDisplayCookieData) {
    this.name = name;
    this.value = value;
    this.isDomain = isDomain;
    this.path = path;
    this.isSecure = isSecure;
    this.expires = expires;
    this.policy = policy;
  }
}

function loadPermissions()
{
  var count = 0;
  var permText;
  var showPolicyField = false;
  var enumerator = gDisplayCookieData ? cookiemanager.enumerator : permissionmanager.enumerator;
  var useDefault = permissionsBundle.formatStringFromName('pmUseDefault', [prefPanelNames[typeID]], 1);
  var cannotStr = permissionsBundle.formatStringFromName('pmCannotStr', [prefPanelNames[typeID]], 1);
  var canStr = permissionsBundle.formatStringFromName('pmCanStr', [prefPanelNames[typeID]], 1);
  var canSession = permissionsBundle.GetStringFromName('pmCanSession');
  var willRedirect = permissionsBundle.formatStringFromName('pmWillRedirect', [prefPanelNames[typeID]], 1);
  var requireAction = permissionsBundle.formatStringFromName('pmRequireAction', [prefPanelNames[typeID]], 1);
  // Unhide name column and splitter in case we're displaying cookie data 
  document.getElementById("nameCol").setAttribute("hidden", !gDisplayCookieData);
  document.getElementById("nameColSplitter").setAttribute("hidden", !gDisplayCookieData);

  while (enumerator.hasMoreElements()) {
    var nextPermission = enumerator.getNext();

    nextPermission = gDisplayCookieData ? nextPermission.QueryInterface(Components.interfaces.nsICookie) : 
                                          nextPermission.QueryInterface(Components.interfaces.nsIPermission);

    var host = nextPermission.host;

    if (gDisplayCookieData) {
      if (nextPermission.policy != nsICookie.POLICY_UNKNOWN)
        showPolicyField = true;
      permissions[count] = new Permission(count++, host, (host.charAt(0)==".") ? host.substring(1, host.length) : host, 
                                          -1, getStatusString(nextPermission.status), nextPermission.name, 
                                          nextPermission.value, nextPermission.isDomain, nextPermission.path, 
                                          nextPermission.isSecure, nextPermission.expires, nextPermission.policy);
    }
    else if (nextPermission.type == dialogType) {
      var capability = nextPermission.capability;
      // capability: 0 = Default (based on popup blocker pref) 1 = Allow / 2 = Block / 3 = Redirect to new tab / 4 = Selection Dialog / 8 = Session only cookie
      if (capability == ALLOW_SESSION_ONLY)
        permText = canSession;
      else if (capability < 3)
        permText = (capability == 0) ? useDefault : (capability == 1) ? canStr : cannotStr;
      else 
        permText = (capability == 3) ? willRedirect : requireAction;

      permissions[count] = new Permission(count++, host, (host.charAt(0)==".") ? host.substring(1, host.length) : host, nextPermission.type, permText);
    }
  }
  // show policy field if at least one cookie has a policy
  if (showPolicyField)
    document.getElementById("policyField").removeAttribute("hidden");
}

function loadTree()
{
  permissionsTreeView.rowCount = permissions.length;
  permissionsTree.treeBoxObject.view = permissionsTreeView;

  if (permissions.length == 0 || permissionsTree.currentIndex > (permissions.length-1))
    permissionsTree.currentIndex = -1;
}

function getConfirmation(aWindow, aTitle, aText, aCheckboxText)
{
  gPreventNewCookies = false;
  var checkbox = {};
  var checkboxText = aCheckboxText ? permissionsBundle.GetStringFromName(aCheckboxText) : null;
  var result = promptService.confirmEx(aWindow, permissionsBundle.GetStringFromName(aTitle), 
                                        permissionsBundle.GetStringFromName(aText),
                                        (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES) + 
                                        (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_NO),
                                        null, null, null, checkboxText, checkbox);
  gPreventNewCookies = checkbox.value;
  return (result == 0); // Yes returns 0 / No, Esc and X returns 1
}

function deletePermission()
{
  var text = gDisplayCookieData ? 'pmConfirmCookieRemovalText' : 'pmConfirmRemovalText';
  var checkboxText = gDisplayCookieData ? 'pmConfirmCookieRemovalCheckbox' : null;

  if (!getConfirmation(window, 'pmConfirmRemovalTitle', text, checkboxText))
    return;

  gUpdatingBatch = "perm-changed";
  DeleteSelectedItemFromTree(permissionsTree, permissionsTreeView, permissions, 
                             deletedPermissions, "dummy-button", "dummy-button");

  if (gDisplayCookieData) { // Remove Cookies
    for (var c = 0; c < deletedPermissions.length; c++)
      cookiemanager.remove(deletedPermissions[c].host, deletedPermissions[c].name, 
                           deletedPermissions[c].path, gPreventNewCookies);
  }  
  else { // Remove Permissions
    for (var p = 0; p < deletedPermissions.length; p++)
      permissionmanager.remove(deletedPermissions[p].host, deletedPermissions[p].type);
  }
  deletedPermissions.length = 0;
  gUpdatingBatch = "";
}

function handlePermissionKeyPress(e)
{
  if (e.keyCode == 46) // delete
    deletePermission();
}

function permissionColumnSort(column, updateSelection)
{
  lastPermissionSortAscending = SortTree(permissionsTree, permissionsTreeView, permissions, 
                                         column, lastPermissionSortColumn, lastPermissionSortAscending, 
                                         updateSelection);
  lastPermissionSortColumn = column;
}

function doHelpButton(aCloseFlag)
{
  var tabbrowser = null;
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1']
                                .getService(Components.interfaces.nsIWindowMediator);
  try { // Catch "undefined xyz" JS errors
    tabbrowser = windowManager.getMostRecentWindow("navigator:browser").getBrowser();
  } catch(ex) {
    window.open();
    tabbrowser = windowManager.getMostRecentWindow("navigator:browser").getBrowser();
  }
  var helpURL = permissionsBundle.GetStringFromName('pmHelpURL');
  // Are we being called from the settings window?
  if (aCloseFlag) { // No, main window
    tabbrowser.loadURI(helpURL);
    window.close();
  }
  else { // Yes, load help/faq page and point to settings section
    tabbrowser.loadURI(helpURL + "#settings");
  }
}

// Used from 'Back' button on permissionManager.xul
function doBackButton()
{
  if (gCookieDataFromPrefPanel && gDisplayCookieData)
    openPrefOverlay(); // Cookie Data -> Pref Panel
  else
    displayCookiePermissions(); // Cookie Data -> Cookie Permissions
}

function isDuplicate(aHost)
{
  aHost = aHost.replace(/file:/, 'scheme:file');

  for (var i = 0; i < permissions.length; i++) {
    if (permissions[i].host == aHost)
      return i;
  }
  return -1;
}

function addPermission(aPermission)
{
  if (aPermission) {
    addHostPermission(permissions[permissionsTree.currentIndex].host, aPermission);
  }
  else { // Add button and ENTER in textfield
    var url = newHostField.value;
    var permission = nsIPermissionManager.DENY_ACTION;

    switch(typeID) {
      case 0: permission = pref.getIntPref("network.cookie.cookieBehavior");
              break;
      case 1: permission = pref.getIntPref("network.image.imageBehavior");
              break;
      case 2: permission = pref.getBoolPref("dom.disable_open_during_load") ? 2 : 1;
              break;
      default:
              permission = pref.getIntPref("multizilla.content-blocker.behavior." + permissionTypes[typeID]);
    }
    permission = (permission == 2) ? nsIPermissionManager.ALLOW_ACTION : nsIPermissionManager.DENY_ACTION;

    if (typeID == 0) { // Check 'lifetimePolicy for Cookies
      var lifetimePolicy;

      try { // This pref setting is only available in mozilla build 20040309 and up!
        lifetimePolicy = pref.getIntPref("network.cookie.lifetimePolicy");
      } catch(ex) {
      }
      permission = (lifetimePolicy == 2) ? ALLOW_SESSION_ONLY : permission;
    }
    else if (typeID != 2) { // Skipped for Cookies & Popups
      var uri = Components.classes['@mozilla.org/network/standard-url;1'].createInstance(Components.interfaces.nsIURI);
      uri.spec = url;
      permission = isFiltered(uri, permissionTypes[typeID]) ? nsIPermissionManager.ALLOW_ACTION : nsIPermissionManager.DENY_ACTION;
    }
    addHostPermission(url, permission);
    permissionsTree.currentIndex = isDuplicate(url);
    doSelect('none');
    permissionsTree.view.selection.select(permissionsTree.currentIndex);
    permissionsTree.treeBoxObject.ensureRowIsVisible(permissionsTree.currentIndex);
    // Clear entry field and disable Add Button
    gLastHost = "";
    newHostField.value = "";
    updateLastHost();
    permissionsTree.focus();
  }
}

function addHostPermission(aHost, aPermission)
{
  loadTree();
  var isFileScheme = aHost.match(/(scheme:file|file:)/i);
  aHost = aHost.replace(/^\s*([-\w]*:\/+)?/, ""); // trim any leading space and scheme
  var URL = isFileScheme ? 'file:' : 'http://' + aHost;
  const gIOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var URI = gIOService.newURI(URL, null, null);

  if (aPermission == -1) { // Toggle permission (true=1/false=0)
    var capability = permissionmanager.testPermission(URI, dialogType);
    aPermission = (capability == nsIPermissionManager.ALLOW_ACTION) ? nsIPermissionManager.DENY_ACTION : nsIPermissionManager.ALLOW_ACTION;
  }
  permissionmanager.add(URI, dialogType, aPermission);
  loadTree();

  if (permissions.length > 0)
    permissionColumnSort('rawHost', false);
}

function switchTree(dialogType)
{
  permissions = [];
  lastPermissionSortColumn = "";
  lastPermissionSortAscending = false;
  setupIDs(dialogType, false);
  // Are we displaying cookie data?
  gDisplayCookieData = (dialogType == "cookie" && gDisplayCookieData);
  gLastCookie = null;
  // If yes, hide 'Toggle' and unhide 'Session' button
  toolbarButtons[TOGGLE_BUTTON].hidden = (dialogType == "cookie");
  toolbarButtons[SESSION_BUTTON].hidden = (dialogType != "cookie");
  document.getElementById('backButton').hidden = (!gDisplayCookieData); // true;

  // Hide entry field and display cookie data box when needed
  document.getElementById('cookieDataBox').hidden = !gDisplayCookieData;
  entryBox.hidden = (dialogType == "cookie" && gDisplayCookieData);
  document.getElementById('storedCookiesButton').hidden = (dialogType != "cookie" || gDisplayCookieData);

  loadPermissions();
  loadTree();
  permissionsTree.focus();
  permissionsTree.view.selection.select(-1);
  var hostFoundOnRow = -1;

  if (permissionsTreeView.rowCount > 0) {
    permissionColumnSort('rawHost', false);

    if (gLastHost) {
      hostFoundOnRow = isDuplicate(gLastHost);

      if (hostFoundOnRow >= 0) {
        newHostField.value = "";
        toolbarButtons[ADD_BUTTON].disabled = true;
        permissionsTree.currentIndex = hostFoundOnRow;
        permissionsTree.view.selection.toggleSelect(permissionsTree.currentIndex);
        permissionsTree.treeBoxObject.ensureRowIsVisible(permissionsTree.currentIndex);
      }
    }
  }
  if (gLastHost && hostFoundOnRow == -1) {
    newHostField.value = gLastHost;
    toolbarButtons[ADD_BUTTON].disabled = gDisplayCookieData;
  }
  selectDefaultButton();
}

function updateTextField()
{
  if (newHostField.value == "") {
    updateToolbarButtons();
    return;
  }
  for (var i = 0; i < inputEnabledButtons.length; i++)
    toolbarButtons[i].disabled = !inputEnabledButtons[i];
}

function updateLastHost()
{
  var currentHost = newHostField.value;

  if (permissionsTreeView.rowCount > 0 && permissionsTree.currentIndex > -1) {
    if (permissionsTree.currentIndex <= permissionsTreeView.rowCount) {
      var selectedRows = GetTreeSelections(permissionsTree);

      if (selectedRows.length == 1)
        gLastHost = permissions[permissionsTree.currentIndex].rawHost || null;
    }
  }
  if (currentHost != gLastHost && newHostField.value != "")
    newHostField.value = "";
  toolbarButtons[ADD_BUTTON].disabled = (entryBox.hidden || newHostField.value == "");
}

function updateToolbarButtons(aInputFlag)
{
  var i = 0;
  var buttons = defaultEnabledButtons;
  var selectedRows = GetTreeSelections(permissionsTree);

  if (permissionsTreeView.rowCount < 1 || permissionsTree.currentIndex < 0 || selectedRows.length == 0) {
    for (i = 0; i < toolbarButtons.length; i++)
      toolbarButtons[i].disabled = true;
  }
  else {
    if (selectedRows.length > 1)
      buttons = selectEnabledButtons;
    else if (dialogType == 'popup') {
      var redirectorEnabled = pref.getBoolPref('multizilla.window-open.redirector');
      popupEnabledButtons[REDIRECT_BUTTON] = redirectorEnabled;
      popupEnabledButtons[ASKME_BUTTON] = redirectorEnabled;
      buttons = popupEnabledButtons;
    }
    else if (dialogType == 'cookie')
      buttons = gDisplayCookieData ? cookieDataEnabledButtons : cookieEnabledButtons;
    // Disable buttons that shouldn't be enabled at this moment
    for (i = 0; i < buttons.length; i++)
      toolbarButtons[i].disabled = !buttons[i];
  }
  toolbarButtons[PREF_BUTTON].disabled = false; // The pref button should always be enabled!
}

function mzGetStringBundle(aPath)
{
  var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
                                   .getService(Components.interfaces.nsIStringBundleService); 
  return strBundleService.createBundle(aPath);
}

function doSelect(aSelection)
{
  switch (aSelection) {
    case "all": // Called from the context menu, or Ctrl+A ?
         permissionsTree.view.selection.selectAll();
         break;
    case "none": // Called from the context menu, or Ctrl+Z ?
         permissionsTree.view.selection.clearSelection();
         permissionsTree.view.selection.select(permissionsTree.currentIndex);
         break;
  }
  permissionsTree.focus();
}

function initContextMenu(aEvent)
{
  var row = permissionsTree.treeBoxObject.getRowAt(aEvent.pageX, aEvent.pageY);
  var menuItems = aEvent.originalTarget.childNodes;

  menuItems[4].setAttribute('label', permissionsBundle.formatStringFromName('pmBlock', [prefPanelNames[typeID]], 1));
  menuItems[5].setAttribute('label', permissionsBundle.formatStringFromName('pmAllow', [prefPanelNames[typeID]], 1));
  menuItems[6].setAttribute('label', permissionsBundle.formatStringFromName('pmSession', [prefPanelNames[typeID]], 1));
  var removeLabelText = gDisplayCookieData ? permissionsBundle.GetStringFromName('pmDeleteCookies') : permissionsBundle.GetStringFromName('pmDelete');
  menuItems[11].setAttribute('label', removeLabelText);
  
  var hidden = (dialogType != "popup");
  // Hide 'Toggle Site Permission' for 'Popups' context menu (display for the rest)
  menuItems[3].hidden = !hidden;
  // Display the 'Popups/Cookies' specific menu items in the context menu
  for (var i = 4; i < 10; i++) {
    menuItems[i].hidden = hidden;
  }
  // Hide 'Allow session Cookies from this site' 
  menuItems[6].hidden = true;
  // Unhide menu items for Cookies
  if (dialogType == "cookie") {
    menuItems[3].hidden = true;
	
    for (i = 4; i < 7; i++)
      menuItems[i].hidden = gDisplayCookieData;
  }
  // Display <menuseparator> based on 'gDisplayCookieData'
  menuItems[10].hidden = gDisplayCookieData;
  var rows = permissionsTree.treeBoxObject.view.rowCount;
  var selectedRows = GetTreeSelections(permissionsTree);
  // Disable "Select All" when there's a selection
  var selectAll = document.getElementById("selectAll");
  // Disable "Deselect All" when there's no selection
  var deselectAll = document.getElementById("undoSelection");

  if (selectedRows.length > 1) {
    selectAll.setAttribute("disabled", "true");
    deselectAll.removeAttribute("disabled");

    for (i = 3; i < 10; i++) {
      menuItems[i].setAttribute("disabled", true);
    }
    return;
  }
  else { // No selection, enable "Select All" and disable "Deselect All"
    if (rows > 1)
      selectAll.removeAttribute("disabled");
    deselectAll.setAttribute("disabled", "true");
  }
  // Disable all menu items for clicks on blank space in the permissions tree!
  if (row == -1) {
    for (i = 2; i < menuItems.length; i++)
      menuItems[i].setAttribute("disabled", true);
  }
  else { // Reenable context menu items
    for (i = 2; i < menuItems.length; i++)
      menuItems[i].removeAttribute("disabled");
  }
  if (dialogType == "popup") {
    // Enable MultiZilla's 'New Window Redirector' menu items based on pref setting
    var redirectorEnabled = pref.getBoolPref('multizilla.window-open.redirector');
    menuItems[8].setAttribute("disabled", redirectorEnabled ? 'false' : 'true'); // Redirect Popups to new tabs
    menuItems[9].setAttribute("disabled", redirectorEnabled ? 'false' : 'true'); // Ask me before opening Popups
  }
}

function openPrefOverlay()
{
  gDisplayCookieData = false;
  gCookieDataFromPrefPanel = false;
  document.getElementById('backButton').hidden = true;
  var winFeatures = "chrome, dependent, modal, dialog" + (gUseStackedWindows ? "" : ", resizable=yes, centerscreen");
  window.openDialog("chrome://multiviews/content/permission-manager/permissionManagerPrefWindow.xul", 
                    "prefWindow", winFeatures, window, "");
}

function getExpiresString(expires)
{
  if (expires) {
    var date = new Date(1000 * expires);
    return gDateService.FormatDateTime("", gDateService.dateFormatLong, gDateService.timeFormatSeconds, 
                                       date.getFullYear(), date.getMonth()+1, date.getDate(), 
                                       date.getHours(), date.getMinutes(), date.getSeconds());
  }
  return cookieBundle.GetStringFromName("AtEndOfSession");
}

function getStatusString(status)
{
  switch (status) {
    case nsICookie.STATUS_ACCEPTED:
      return cookieBundle.GetStringFromName("accepted");
    case nsICookie.STATUS_FLAGGED:
      return cookieBundle.GetStringFromName("flagged");
    case nsICookie.STATUS_DOWNGRADED:
      return cookieBundle.GetStringFromName("downgraded");
  }
  return "";
}

function getPolicyString(policy)
{
  switch (policy) {
    case nsICookie.POLICY_NONE:
      return cookieBundle.GetStringFromName("policyUnstated");
    case nsICookie.POLICY_NO_CONSENT:
      return cookieBundle.GetStringFromName("policyNoConsent");
    case nsICookie.POLICY_IMPLICIT_CONSENT:
      return cookieBundle.GetStringFromName("policyImplicitConsent");
    case nsICookie.POLICY_EXPLICIT_CONSENT:
      return cookieBundle.GetStringFromName("policyExplicitConsent");
    case nsICookie.POLICY_NO_II:
      return cookieBundle.GetStringFromName("policyNoIICollected");
  }
  return "";
}

function cookieSelected()
{
  var selectedRows = GetTreeSelections(permissionsTree);

  if (permissionsTreeView.rowCount < 1 || permissionsTree.currentIndex < 0 || selectedRows.length == 0 || selectedRows.length > 1) {
    if (gLastCookie)
      clearCookieProperties();
    return;
  }
  var selectedRow = selectedRows[0];

  if (gLastCookie == selectedRow)
    return;
  gLastCookie = selectedRow;
  var value, field, isSecureText = permissions[selectedRow].isSecure ? permissionsBundle.GetStringFromName("pmEncrypted") : permissionsBundle.GetStringFromName("pmUnencrypted");

  var props = [
    {id: "ifl_name", value: permissions[selectedRow].name},
    {id: "ifl_value", value: spliceCookieData(permissions[selectedRow].value, 70)}, 
    {id: "ifl_isDomain", value: permissions[selectedRow].isDomain ? cookieBundle.GetStringFromName("domainColon") : cookieBundle.GetStringFromName("hostColon")},
    {id: "ifl_host", value: permissions[selectedRow].host},
    {id: "ifl_path", value: permissions[selectedRow].path},
    {id: "ifl_isSecure", value: isSecureText},
    {id: "ifl_expires", value: getExpiresString(permissions[selectedRow].expires)},
    {id: "ifl_policy", value: getPolicyString(permissions[selectedRow].policy)}
  ];

  for (var i = 0; i < props.length; i++)
  {
    field = document.getElementById(props[i].id);
    field.value = props[i].value;
  }
}

function spliceCookieData(str, length) 
{
  var start = 0;
  var result = "";

  while (start < str.length) {
    result += str.slice(start, (start+length)) + " ";
    start += length;
  }
  return result;
}

function clearCookieProperties()
{
  var properties = ["ifl_name", "ifl_value", "ifl_host", "ifl_path", "ifl_isSecure", "ifl_expires", "ifl_policy"];

  for (i in properties)
    document.getElementById(properties[i]).value = "";
}

function displayCookieData()
{
  gDisplayCookieData = true;
  switchTree("cookie");

  if (permissionsTree.currentIndex == -1)
    clearCookieProperties();
}

function displayCookiePermissions()
{
  gDisplayCookieData = false;
  gCookieDataFromPrefPanel = false;
  switchTree("cookie");
}

function selectDefaultButton()
{
  var defaultButton;
  var backButton = document.getElementById('backButton');
  var acceptButton = document.documentElement.getButton('accept');
  var cancelButton = document.documentElement.getButton('cancel');

  backButton.removeAttribute('default');
  acceptButton.removeAttribute('default');
  cancelButton.removeAttribute('default');

  if (backButton.hidden)
    defaultButton = acceptButton;
  else
    defaultButton = backButton;
  defaultButton.setAttribute('default', 'true');
  // defaultButton.focus();
}


function selectFile(aWindow, title, mode)
{
  gFileName = null; // reset global filename
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

  mode = (mode == 'open') ? nsIFilePicker.modeOpen : nsIFilePicker.modeSave;

  filePicker.init(aWindow, title, mode);
  filePicker.appendFilter("RDF", "*.rdf");
  filePicker.defaultString = "content-blocker.rdf";
  var filePickerState = filePicker.show()

  if (filePickerState != nsIFilePicker.returnCancel) {
    if (filePicker.fileURL.spec && filePicker.fileURL.spec.length > 0)
      gFileName = filePicker.fileURL.spec;
  }
  return filePickerState;
}

function exportFilters(aWindow, title, mode)
{
  var state = selectFile(aWindow, title, mode);

  if (state == 1) // Cancel
    return;
  else if (state == 2) { // Ok, overwrite existing file, ask for confirmation
    if (!getConfirmation(aWindow, 'cbExportManager', 'cbExportConfirm'))
      return;
  }
  if (gFileName.match('.rdf') == null)
    gFileName += ".rdf";
  cbDatasource = getNewOrCurrentDSByFilename(gFileName, true, null, true);
  var saved = 0;
  // Scan through the array for valid type numbers (skips cookies/popups)
  for (var i = 0; i < patternTypeNumbers.length; i++) {
    var index = patternTypeNumbers[i];
    var resourceID = "multizilla:content-blocker:" + permissionTypes[index];
    var patterns = getPatternPrefFor(permissionTypes[index]);

    if (patterns.length > 1)
      saved += writePatternsFor(permissionTypes[index], resourceID, patterns, state);
  }
  title = permissionsBundle.GetStringFromName('cbExportManager');
  var text = "A total of [ " + saved + " ] filters are saved in the specified file";
  promptService.alert(aWindow, title, text);
  cbDatasource = null;
}

function writePatternsFor(type, resourceID, patterns, overWriteFlag)
{
  var index = 0
  var patternCount = patterns.length;
  // alert(type + "\n" + resourceID + "\n" + patternCount + "\n" + overWriteFlag);
  if (overWriteFlag == 2)
    removeEntriesFor(type);
		
  try {
    var entries = mzConstructRDFContainer(cbDatasource, 'seq', resourceID);

    for (; index < patternCount; index++) {
      var property = RDF.GetLiteral(patterns[index]);
      entries.InsertElementAt(property, (index + 1), true);
    }
  } catch(ex) {
    // continue
  }
  cbDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
  return(index);
}

function getPatternPrefFor(type)
{
  var patterns = new Array();

  try {
    var prefValue = pref.getComplexValue(prefPatternLeader + type, Components.interfaces.nsISupportsString);
    patterns = prefValue.data.replace(/\s/g, '').split(',');
    patterns = patterns.sort();
  } catch(ex) {
    // die silently
  }
  return patterns;
}

function cbEntries(aResourceID)
{
  var entries = mzRDFCUtils.MakeSeq(cbDatasource, RDF.GetResource(aResourceID));

  if (!entries)
    return null;
  if (!entries.GetElements())
    return null;
  return entries;
}

function removeEntriesFor(aPermissionType)
{
  var resourceIDs = new Array();
  resourceIDs.push("urn:" + aPermissionType);
  resourceIDs.push("multizilla:content-blocker:" + aPermissionType);

  for (var i = 0; i < resourceIDs.length; i++) {
    var resource = RDF.GetResource(resourceIDs[i]);
    // alert(resourceIDs[i]);
    if (mzRDFCUtils.IsEmpty(cbDatasource, resource))
      continue; // Skip empty Seq's
    else if (mzRDFCUtils.IsSeq(cbDatasource, resource)) {
      var index = cbEntries(resourceIDs[i]).GetCount();

      for (index; index > 0; index--)
        cbEntries(resourceIDs[i]).RemoveElementAt(index, true);
    }
  }
  cbDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
}

function importFilters(aWindow, title, mode)
{
  var gStatus = selectFile(aWindow, title, mode);

  if (gStatus == 1) // Cancel
    return;
  if (gFileName.match('.rdf') == null)
    gFileName += ".rdf";

  cbDatasource = getNewOrCurrentDSByFilename(gFileName, false, null, true);
  var dsStats = getDataSourceStatistics();

  if (dsStats == null) {
    promptService.alert(aWindow, permissionsBundle.GetStringFromName('cbImportFileErrorTitle'),
                        permissionsBundle.GetStringFromName('cbImportFileErrorText'));
    return;
  }
  var winFeatures = "chrome, dependent, modal, dialog" + (gUseStackedWindows ? "" : ", resizable=yes, centerscreen");
  aWindow.openDialog("chrome://multiviews/content/permission-manager/permissionManagerImportExport.xul", 
                     "ImportExportindow", winFeatures, window, dsStats);
}

function getDataSourceStatistics()
{
  var failures = 0;
  var dsStatistics = new Array();

  for (var i = 0; i < patternTypeNumbers.length; i++) {
    var index = patternTypeNumbers[i];
    var resourceID = "multizilla:content-blocker:" + permissionTypes[index];
    var resource = RDF.GetResource(resourceID);
    var seqStats = new Object();

    seqStats.typeID = permissionTypes[index];
    seqStats.resourceID = resourceID;

    if (mzRDFCUtils.IsSeq(cbDatasource, resource))
      seqStats.entries = cbEntries(resourceID).GetCount();
    else {
      seqStats.entries = 0;
      failures++;
    }
    dsStatistics.push(seqStats);
  }
  return (failures == 7) ? null : dsStatistics;
}

function isFiltered(uri, type)
{
  var patterns = new Array();
  var url = uri.spec.match(/^www./i) ? uri.spec : "." + uri.spec;

  try {
    var pattern = pref.getComplexValue("multizilla.content-blocker.pattern.for." + type, Components.interfaces.nsISupportsString);
    patterns = pattern.data.replace(/\s/g, '').replace(/\./g, '\.+').replace(/\*/g, '.*');
    patterns = patterns.split(',');
    patterns = patterns.sort();
  } catch(ex) {
    return false;
  }
  if (patterns.length > 0) {
    for (var index = 0; index < patterns.length; index++) {
      var regexp = new RegExp();

      if (url.match(regexp.compile(patterns[index]),'i'))
        return true;
    }
  }
  return false;
}

function initWindowOpenType()
{
  try {
    gUseStackedWindows = pref.getBoolPref("multizilla.managers.use-stacked-windows");
  } catch(ex) {
    gUseStackedWindows = /Win/.test(navigator.platform);
  }
}
