/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - November 20, 2003.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2003-2004
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
*/

const MZ_TRUE = "true";
const MZ_FALSE = "false";

var gDialogManager, gHeight, gTabbrowser, gModified;
var gBackLabel, gBackImage, gForwardLabel, gForwardImage;
var reloadInterval, helpPages, dataCollection = null;
var elementArray = new Array();
var elementIDs = new Array("label", "custom", "firstpage", "image", "location", "interval", 
                           "selected", "allowPlugins", "allowJavascript", "allowMetaRedirects", 
                           "allowSubframes", "allowImages", "interval-enabled", "allow-all", 
                           "suppress-all");
var defaultValues = ["about:blank", "false", "about:blank", "", "about:blank", "0", "false", "true", "true", "true", "true", "true"];

function Startup()
{
  gDialogManager = window.arguments[0];
  gHeight = (gDialogManager.innerHeight / 50 - 2);
  gTabbrowser = gDialogManager.getTabbrowser(true);
  gBackImage = document.getElementById('BackImage');
  gBackLabel = document.getElementById('BackLabel');
  gForwardLabel = document.getElementById('ForwardLabel');
  gForwardImage = document.getElementById('ForwardImage');
  initTabProperties();
  document.getElementById('mac_button_spring').hidden = !(/Mac/.test(navigator.platform));
  selectDefaultButton();
  setResource(0);

  if (gDialogManager.gUseStackedWindows) {
    document.documentElement.setAttribute("hidechrome", "true");
    window.outerWidth = gDialogManager.innerWidth;
    window.outerHeight = 0;
    slideOpen(0);
  }
  else {
    window.outerWidth = gDialogManager.outerWidth;
    window.outerHeight = gDialogManager.outerHeight;
    document.getElementById("main-box").collapsed = false;
  }
}

function slideOpen(aStep)
{
  if (aStep < gHeight) {
    window.outerHeight += 50;
    setTimeout(slideOpen, 20, ++aStep);
  } 
  else {
    /* var defaultHeight = gDialogManager.document.documentElement.getAttribute('inner-height');

    if (gDialogManager.innerHeight < defaultHeight)
      gDialogManager.innerHeight = defaultHeight; */
    window.outerHeight = gDialogManager.innerHeight;
    document.getElementById("main-box").collapsed = false;
  }
}

function slideClosed(aStep)
{
  if (aStep < gHeight) {
    window.outerHeight -= 50;
    setTimeout(slideClosed, 10, ++aStep);
  } 
  else
    window.close();
}

function initTabProperties()
{
  // Is this the first run?
  if (typeof elementIDs[0] == 'string') {
    // Yes, initialize element array (used in activateSettings)
    for (i in elementIDs)
      elementArray[i] = document.getElementById(elementIDs[i]);
  }
  if (mzRDFStore == null)
    initializeRDFStorageFile();
  // Initialize textboxes, checkboxes and buttons
  dataCollection = getTabDataFromResource(RDF.GetResource(gDialogManager.gResource));

  if (dataCollection) {
    activateSettings(dataCollection);
    // Initialize the 'Allow All' or 'Suppress All' buttons
    onProperty(null);
  }
  setBackForwardButtons();
}

function activateSettings(aDataCollection)
{
  elementArray[0].value = aDataCollection[0].value; // title
  elementArray[1].checked = (aDataCollection[1].value == MZ_TRUE); // custom
  elementArray[2].value = aDataCollection[2].value; // firstpage
  elementArray[3].value = aDataCollection[3].value; // image
  elementArray[4].value = aDataCollection[4].value; // location
  reloadInterval = aDataCollection[5].value; // interval
  elementArray[6].checked = (aDataCollection[6].value == MZ_TRUE) ? false : true;
  elementArray[12].checked = (reloadInterval > 0);
  elementArray[5].disabled = (reloadInterval == 0);
  gTabbrowser.initReloadPopup(elementArray[5].firstChild, null, reloadInterval);
  // Tab Properties
  for (var i = 7; i < elementArray.length-3; i++) {
    elementArray[i].checked = (aDataCollection[i].value == MZ_TRUE);
  }  
}

function getDataCollection()
{
  var defaultScheme = "http://";
  var aDataCollection = new Array();
  // Data Collection buildup
  aDataCollection[0] = {value:elementArray[0].value}; // title
  aDataCollection[1] = {value:elementArray[1].checked ? MZ_TRUE : MZ_FALSE}; // custom
  // Check for missing scheme in 'firstpage' textbox.
  if (elementArray[2].value.indexOf(":") < 0) // Add default scheme
    elementArray[2].value = defaultScheme + elementArray[2].value;
  if (elementArray[2].value == defaultScheme)
    elementArray[2].value = defaultValues[2];
  aDataCollection[2] = {value:elementArray[2].value}; // firstpage
  aDataCollection[3] = {value:elementArray[3].value}; // image
  // Check for missing scheme in 'location' textbox.
  if (elementArray[4].value.indexOf(":") < 0) // Add default scheme
    elementArray[4].value = defaultScheme + elementArray[4].value;
  if (elementArray[4].value == defaultScheme)
    elementArray[4].value = defaultValues[4];
  aDataCollection[4] = {value:elementArray[4].value}; // location
  aDataCollection[5] = {value:reloadInterval}; // Reload interval
  aDataCollection[6] = {value:elementArray[6].checked ? MZ_FALSE : MZ_TRUE};
  // Tab Properties
  for (var i = 7; i < 12; i++) {
    aDataCollection[i] = {value:elementArray[i].checked ? MZ_TRUE : MZ_FALSE};
  }
  return aDataCollection;
}

function refreshReloadState()
{
  var disabled = !elementArray[12].checked
  elementArray[5].disabled = disabled;

  if (disabled) {
    elementArray[5].value = 0;
    reloadInterval = 0;
    hasModifiedSettings();
  }
}

function onProperty(aElement)
{
  if (aElement) // skipped at startup
    hasModifiedSettings();
  var id = aElement ? aElement.getAttribute("id") : "";
  var properties = 0;
  // Count number of checked property checkboxes
  for (var i = 7; i < 12; i++) {
    if (elementArray[i].checked)
      properties++
  }
  // Called from the 'Allow All' or 'Suppress All' buttons
  if (id == elementIDs[13] || id == elementIDs[14]) {
    properties = (id == elementIDs[13]) ? 5 : 0;
    for (i = 7; i < 12; i++)
      elementArray[i].checked = properties ? true : false;
  }
  // Update button state
  elementArray[13].disabled = (properties == 5) ? true : false;
  elementArray[14].disabled = (properties == 0) ? true : false;
}

function setReloadInterval(aMenuitem)
{
  hasModifiedSettings();
  reloadInterval = (aMenuitem.getAttribute('value') * 1000);
}

function onHelp()
{
  if (!helpPages) {
    var stringBundle = document.getElementById("properties");
    helpPages = stringBundle.getString("helpPages-SessionManager");
    stringBundle = null; // Cleanup
  }
  gTabbrowser.loadURI(helpPages);
  window.close();
}

function onCancel()
{
  slideClosed(0);
}

function onAccept()
{
  storeDataCollection();
  slideClosed(0);
}

function storeDataCollection()
{
  var MZ = "http://multizilla.mozdev.org/rdf#";
  var modifiedDataCollection = getDataCollection();
  var resource = RDF.GetResource(gDialogManager.gResource);

  try {
    for (var i = 0; i < 12; i++) {
      if (dataCollection[i].value != modifiedDataCollection[i].value) {
        var property = RDF.GetResource(MZ+elementIDs[i]);
        var newValue = RDF.GetLiteral(modifiedDataCollection[i].value);
        var currentValue = mzRDFStore.GetTarget(resource, property, true);

        if (modifiedDataCollection[i].value == defaultValues[i])
          mzRDFStore.Unassert(resource, property, currentValue);
        else
          addOrChangeValue(resource, property, currentValue, newValue);
      }
    }
  } catch(ex) { 
    // die silently
  }
  mzRDFStore.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
  gModified = false;
}

function hasModifiedSettings(aState)
{
  var acceptButton = document.documentElement.getButton('accept');
  gModified = (aState == "no") ? false : true;
  acceptButton.disabled = !gModified;
  var cancelButton = document.documentElement.getButton('cancel');
  cancelButton.label = gDialogManager.stringBundle.getString('prefPanelCancelButton.label');
  cancelButton.accessKey = gDialogManager.stringBundle.getString('prefPanelCancelButton.accesskey');
  selectDefaultButton();
}

function selectDefaultButton()
{
  var acceptButton = document.documentElement.getButton('accept');
  var cancelButton = document.documentElement.getButton('cancel');
  var defaultButton = cancelButton;

  acceptButton.removeAttribute('default');
  cancelButton.removeAttribute('default');

  if (gModified)
    defaultButton = acceptButton;
  else
    acceptButton.disabled = true;
  defaultButton.setAttribute('default', 'true');
  defaultButton.focus();
}

function setBackForwardButtons()
{
  var tree = gDialogManager.tree;
  var row = tree.currentIndex;
  var rows = (tree.treeBoxObject.view.rowCount-1);
  var disabled = (row > 0) ? tree.treeBoxObject.view.isContainer(row - 1) : false;

  gBackLabel.disabled = disabled;
  gBackImage.setAttribute('disabled', disabled ? 'true' : 'false');
  disabled = (row == rows) ? true : ((row + 1) < rows) ? tree.treeBoxObject.view.isContainer(row + 1) : false;
  gForwardLabel.disabled = disabled;
  gForwardImage.setAttribute('disabled', disabled ? 'true' : 'false');
}

function setResource(aDirection)
{
  var tree = gDialogManager.tree;
  tree.currentIndex = tree.currentIndex + aDirection;
  gDialogManager.sessionManager.selectRows('none');
  tree.treeBoxObject.ensureRowIsVisible(tree.currentIndex);

  var resource = tree.builderView.getResourceAtIndex(tree.currentIndex).Value;
  document.getElementById('TabSessionID').value = resource;
  gDialogManager.gResource = resource;
}

function refreshPropertyData(aDirection)
{
  // We need to check the disable state of the 'label' buttons!
  var canBack = gBackLabel.disabled;
  var canForward = gForwardLabel.disabled;

  if (aDirection == -1 && canBack || aDirection == 1 && canForward)
    return;

  var shouldSkip = false;
  var prefString = "multizilla.tabs.session-property-changes.confirmation";

  try {
    shouldSkip = gDialogManager.pref.getBoolPref(prefString);
  } catch (ex) {
    gDialogManager.pref.setBoolPref(prefString, shouldSkip);
  }
  var result = (gModified && shouldSkip) ? 0 : 1;

  if (gModified && !shouldSkip) {
    var dialogTitle = gDialogManager.stringBundle.getString("tsmConfirmActionTitle");
    var dialogText = gDialogManager.stringBundle.getString("tsmConfirmStorageText");
    var checkboxText = gDialogManager.stringBundle.getString("tsmKeepStorageDecisionText");
    var checkbox = {checked:true}
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
    result = promptService.confirmEx(window, dialogTitle, dialogText,
                                     (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES) + 
                                     (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_NO),
                                     null, null, null, checkboxText, checkbox);

    gDialogManager.pref.setBoolPref(prefString, checkbox.value);
  }
  if (result == 0) // Yes returns 0 and No, Esc and X on window returns 1
    storeDataCollection();
  setResource(aDirection);
  gModified = false;
  selectDefaultButton();
  initTabProperties();
}


